#include "qobject.h"
#include "qevent.h"
#include <Fl.H>
#include <math.h>
#include <unistd.h>

QObject::__timerobj * QObject::m_TimerList[MAXTIMERS];
bool QObject::m_bInit = false;

QObject::__timerobj::__timerobj(QObject * p, int interval)
{
  m_pThis = p;
  m_nInterval = interval;
}

void QObject::InitializeTimers()
{
  for(int i=0; i<MAXTIMERS; i++)
    m_TimerList[i] = NULL;
  m_bInit = true;
}

QObject::~QObject()
{
  for(int i=0; i<MAXTIMERS; i++)
  {
    if(m_TimerList[i])
    {
      if(m_TimerList[i]->getThis() == this)
      {
	delete m_TimerList[i];
	m_TimerList[i] = NULL;
      }
    }
  }
}

void QObject::killTimer(int id)
{
  DestroyTimer(id);
}

void QObject::TimerFire()
{ 
  QTimerEvent e;
  timerEvent(&e);
}

void QObject::timerEvent(QTimerEvent * e)
{
  cerr << "QObject::timerEvent base called\n";
}

int QObject::startTimer(int interval)
{
  __timerobj * t = CreateTimer(interval); 

  Fl::add_timeout(interval * .001, runthread, t);
  return t->getId();
}

QObject::__timerobj * QObject::CreateTimer(int interval)
{ 
  __timerobj * t = new __timerobj(this,interval);

  if(!m_bInit)
    InitializeTimers();

  for(int i=1; i<MAXTIMERS; i++)
  {
    if(m_TimerList[i] == NULL)
    {
      m_TimerList[i] = t;
      t->setId(i);
      return t;
      break;
    }
  } 
  return NULL;
}	

void QObject::DestroyTimer(int timer)
{
  if(m_TimerList[timer])
  {
    Fl::remove_timeout(runthread,m_TimerList[timer]);
    delete m_TimerList[timer];
  }
  m_TimerList[timer] = NULL;
}

void QObject::runthread(void * p)
{
  for(int i=0; i<MAXTIMERS; i++)
  {
    if(m_TimerList[i] == p)
    {
      __timerobj * t = (__timerobj*)p;

      if(t->getThis())
	t->getThis()->TimerFire();

      int id = t->getId();
      
      if(m_TimerList[id])
	Fl::add_timeout(t->getInterval() * .001 ,runthread,p);
      break;
    }
  }
}

bool QObject::connect ( const QObject * sender, 
			const char * signal, const QObject * receiver,
			const char * member )
{ 
  if(!sender->m_ConnectionDict[signal+1])
    sender->m_ConnectionDict.insert(signal+1,new QConnectionList());
  
  QConnectionList * cl = sender->m_ConnectionDict[signal+1];
  cl->append(new QConnection(receiver,
			     receiver->findslot(member), member));
  return true;
}

bool QObject::connect ( const QObject * sender, const char * signal, 
			const char * member ) const 
{ 
  return connect(sender,signal,this,member);
}

QMember QObject::findsignal(const char * signal) const
{
  //  cerr << "QObject::findsignal " << className() << "::" << signal << "\n";
  if(metaObject()) {
    QMember mbr = metaObject()->findsignal(signal);
    return mbr;
  } else {
    return 0;
  }
}

QMember QObject::findslot(const char * slot) const
{
  //  cerr << "QObject::findslot " << className() << "::" << slot << "\n";
  if(metaObject()) {
    QMember mbr = metaObject()->findslot(slot);
    return mbr;
  } else {
    return 0;
  }
}

void QObject::activate_signal(const char * signal, const char * p0) 
{
  QConnectionList * cl = m_ConnectionDict[signal];
  
  //  cerr << " QObject::activate_signal " << signal << " cl = " << cl << "\n";
  //  cerr << " QObject::activate_signal this =  " << this << "\n";

  if(!cl)
    return;

  QConnection * c;
  for(int i=0; i<cl->count(); i++) {
    c = (*cl)[i];
    if(c) {
      typedef void (QObject::*RT1)(const char *);
      typedef RT1 * PRT1;

      RT1  qm = *((PRT1)(c->member()));
      if(qm)
	(c->object()->*qm)(p0);
    }
  }
}

void QObject::activate_signal(const char * signal) 
{ 
  QConnectionList * cl = m_ConnectionDict[signal];
  
  // cerr << " QObject::activate_signal " << signal << " cl = " << cl << "\n";
  // cerr << " QObject::activate_signal this =  " << this << "\n";

  if(!cl)
    return;

  QConnection * c;
  for(int i=0; i<cl->count(); i++) {
    c = (*cl)[i];
    if(c) {
      QMember qm = *c->member();
      if(qm)
	(c->object()->*qm)();
    }
  }
}

void QObject::activate_signal(const char * signal, int p0) 
{
  QConnectionList * cl = m_ConnectionDict[signal];
  
  //  cerr << " QObject::activate_signal " << signal << " cl = " << cl << "\n";
  //  cerr << " QObject::activate_signal this =  " << this << "\n";

  if(!cl)
    return;

  QConnection * c;
  for(int i=0; i<cl->count(); i++) {
    c = (*cl)[i];
    if(c) {
      typedef void (QObject::*RT1)(int);
      typedef RT1 * PRT1;

      RT1  qm = *((PRT1)(c->member()));
      if(qm)
	(c->object()->*qm)(p0);
    }
  }
}

QConnectionList * QObject::receivers(const char * signal) const 
{
  QConnectionList * cl = m_ConnectionDict[signal];
  return cl;
}



















