#include "http.h"

#include "WWWLib.h"
#include "WWWHTTP.h"
#include "WWWInit.h"

int printer (const char * fmt, va_list pArgs)
{
  return 0;
    return (vfprintf(stdout, fmt, pArgs));
}

int tracer (const char * fmt, va_list pArgs)
{
  return 0;
    return (vfprintf(stderr, fmt, pArgs));
}

void HTTPConnection::setHTML(KHTMLWidget * p)
{
  m_pHTML = p;
}

HTTPConnection::HTTPRequest::~HTTPRequest()
{
  delete [] m_Buffer;
}

void HTTPConnection::HTTPRequest::addBuffer(unsigned char * buf, int len)
{
  if(!m_Buffer)
    m_Buffer = (unsigned char *)malloc(len + 1);
  else
  {
    unsigned char *p = (unsigned char *)realloc(m_Buffer,m_nSize + len + 1);
    m_Buffer = p;
  }
  memcpy(m_Buffer + m_nSize,buf,len);
  *(m_Buffer + m_nSize + len) = 0;
  m_nSize += len;

#ifdef NEVER
  if(!m_Buffer) {
    m_Buffer = new unsigned char[len];
  } else {
    unsigned char * p  = new unsigned char[m_nSize + len];
    memcpy(p,m_Buffer,m_nSize);
    delete [] m_Buffer;
    m_Buffer = p;
  }
  memcpy(m_Buffer + m_nSize,buf,len);
  m_nSize += len;
#endif
}

HTTPConnection::HTTPConnection() 
{ 
  initMetaObject();

  pthread_mutex_init(&m_ListLock, 0);

  m_bThreadRunning = false;
  m_pDocCallback = 0;

  HTProfile_newPreemptiveClient("ViewML", "0.14");
  
  HTPrint_setCallback(printer);
  HTTrace_setCallback(tracer);
  
#if 0
  HTSetTraceMessageMask("sop");
#endif
  
  /* Register the default set of MIME header parsers */
  HTMIMEInit();
  
}

HTTPConnection::~HTTPConnection()
{
  HTFormat_deleteAll();
  HTLibTerminate();  
  pthread_mutex_destroy(&m_ListLock);
}

int HTTPConnection::Connect(QObject * p)
{
  connect(p,SIGNAL(fileRequest(const char*)),SLOT(slotOpenURL(const char*)));
  connect(p,SIGNAL(URLSelected(const char*,int,const char*)), 
	  SLOT(slotClick(const char*,int,const char*)));
  connect(p,SIGNAL(documentDone()), SLOT(slotDocDone()));
}


void HTTPConnection::slotClick(const char * d1, int b, const char * p)
{
  //  loadPage(d1);
  if(m_pDocCallback)
    m_pDocCallback(d1,0,0);

  m_bDocDone = false;
  StartThread();
  cerr << "slotClick " << d1 << "  " << p << "\n";
}

void HTTPConnection::slotDocDone()
{
  m_bDocDone = true;

  do_size();

}

void HTTPConnection::slotOpenURL(const char * URL)
{
  m_bDocDone = false;

  AddReq(URL);
  StartThread();
}

void HTTPConnection::StartThread()
{
  if(m_bThreadRunning)
    return;
  
  m_bThreadRunning = true;
  pthread_create(&m_Thread,NULL,runthread,this);

  BeginDataTimer();
}

void HTTPConnection::loadPage(const char * URL)
{
  m_bDocDone = false;
  AddReq(URL,true);
  StartThread();
}


void * HTTPConnection::runthread(void * p)
{
  HTTPConnection * pThis = (HTTPConnection*)p;
  HTTPRequest * r = 0;

  r = pThis->GetReq();
  while(r) {
    pThis->getURL(r);
    pThis->AddFinished(r);
    r = pThis->GetReq();
  }

  pthread_detach(pThis->m_Thread);
  pThis->m_bThreadRunning = false;

  return 0;
}

void HTTPConnection::StopThread()
{

}

int  HTTPConnection::getURL(HTTPRequest * r)
{
  const char * URL = r->getURL();
  request = HTRequest_new();
  HTChunk * chunk;
  
  HTRequest_setOutputFormat(request, WWW_SOURCE);
  HTRequest_setPreemptive(request, YES);
  if (URL) {
    char * cwd = HTGetCurrentDirectoryURL();
    char * absolute_url = HTParse(URL, cwd, PARSE_ALL);

    chunk = HTLoadToChunk(absolute_url, request);

    HT_FREE(absolute_url);
    HT_FREE(cwd);

    /* If chunk != NULL then we have the data */
    if (chunk) {
      int sz = HTChunk_size(chunk);
      r->addBuffer((unsigned char*)HTChunk_data(chunk),sz);
      HTChunk_delete(chunk);
    }
 } 
  
  HTRequest_delete(request);
  
  return 0;
}

void HTTPConnection::AddReq(const char * url, bool bNewPage)
{
  HTTPRequest * r = new HTTPRequest(url,bNewPage);
  pthread_mutex_lock(&m_ListLock);
  m_Reqs.push(r);
  pthread_mutex_unlock(&m_ListLock);
}

HTTPConnection::HTTPRequest * HTTPConnection::GetReq()
{
  HTTPRequest * r = 0;
  pthread_mutex_lock(&m_ListLock);
  if(m_Reqs.size()) {
    r = m_Reqs.front();

    m_Reqs.pop();

  }

  if(m_Reqs.size()) {
    StopThread();
  }

  pthread_mutex_unlock(&m_ListLock);
  return r;
}

int HTTPConnection::ReqSize()
{
 pthread_mutex_lock(&m_ListLock);
 int r = m_Reqs.size();
 pthread_mutex_unlock(&m_ListLock);
 return r;

}

void HTTPConnection::AddFinished(HTTPRequest * r)
{
  pthread_mutex_lock(&m_ListLock);
  m_Finished.push(r);
  pthread_mutex_unlock(&m_ListLock);
}

HTTPConnection::HTTPRequest * HTTPConnection::GetFinished()
{
  HTTPRequest * r = 0;
  pthread_mutex_lock(&m_ListLock);
  if(m_Finished.size()) {
    r = m_Finished.front();
    m_Finished.pop();
  }
  pthread_mutex_unlock(&m_ListLock);
  return r;
}

int HTTPConnection::FinishedSize()
{
 pthread_mutex_lock(&m_ListLock);
 int r = m_Finished.size();
 pthread_mutex_unlock(&m_ListLock);
 return r;

}

#include "http.moc"


