
#include <kapp.h>
#include "html.h"
#include <Fl.H>
#include <fl_draw.H>
#include <Fl_Window.H>
#include <Fl_Button.H>
#include <Fl_Pixmap.H>
#include <Fl_Input.H>
#include <Fl_Pack.H>
#include <Fl_Scroll.H>
#include <Fl_Image.H>
#include "http.h"
#include "history.h"
#include "fltk/qlist.h"

QList<QString> g_History;
int g_HistoryIndex=-1;


int g_AppWidth = 640;
int g_AppHeight = 480;

#define APPWIDTH (g_AppWidth)
#define APPHEIGHT (g_AppHeight)

#define TITLEHEIGHT 30

#define SCROLLWIDTH 15
#define SCROLLHEIGHT 15
#define SCROLLPAD 0

#define WIDTH (APPWIDTH - SCROLLWIDTH - SCROLLPAD - SCROLLPAD - SCROLLPAD)
#define HEIGHT (APPHEIGHT - TITLEHEIGHT - SCROLLHEIGHT - SCROLLPAD - SCROLLPAD)

extern Fl_Button *Forward;
extern Fl_Button *Backward;
extern Fl_Button *Stop;
extern Fl_Button *Reload;


static char oldstr[256] = "";
static int hindex = 0;

int g_Vert = 0;
Fl_Input * g_Input;
Fl_Scrollbar * g_VertScroll;
Fl_Scrollbar * g_HorzScroll;
Fl_Window * g_HTMLWindow;
Fl_Window * g_AppWindow;

void LoadPage(const char * URL);
Fl_Window* make_window();
void ResizeWindow(int w, int h);

const QColor color0(0,0,0);
const QColor color1(255,255,255);
const QColor black(0,0,0);
const QColor white(255,255,255);
const QColor darkGray(64,64,64);
const QColor gray(128,128,128);
const QColor lightGray(178,178,178);
const QColor red(255,0,0);
const QColor green(0,255,0);
const QColor blue(0,0,255);
const QColor cyan(128,0,128);
const QColor magenta(128,128,0);
const QColor yellow(0,128,128);
const QColor darkRed(178,64,64);
const QColor darkGreen(64,178,64);
const QColor darkBlue(64,64,178);
const QColor darkCyan(178,64,178);
const QColor darkMagenta(178,178,64);
const QColor darkYellow(64,178,178);

KApplication g_KApp;

const QCursor arrowCursor;      
const QCursor upArrowCursor;    
const QCursor crossCursor;      
const QCursor waitCursor;       
const QCursor ibeamCursor;      
const QCursor sizeVerCursor;    
const QCursor sizeHorCursor;    
const QCursor sizeBDiagCursor;  
const QCursor sizeFDiagCursor;  
const QCursor sizeAllCursor;    
const QCursor blankCursor;      


#define xHTML_WIDGET KHTMLView
#define HTML_WIDGET KHTMLWidget

void ResizeScrollbars(int w, int h);
void LoadDoc(const char * URL, int w, int h);

HTML_WIDGET * g_HTML;
HTTPConnection * g_Conn;

#ifdef _NANOX

// the below is needed to fix some X compilation problems with Nano-X
unsigned long KeyPress;
bool XCheckTypedEvent( Display *, unsigned long , 
		       XEvent * )
{
	return false;
}
#endif

int main(int argc, char ** argv)
{

  Fl_Window * mw;

  g_Conn = new HTTPConnection();
  g_Conn->setDocCallback(LoadDoc);

  mw = make_window();

  g_Conn->setHTML(g_HTML);

  mw->show(argc,argv);

  g_HTML->setGeometry(SCROLLPAD,TITLEHEIGHT ,WIDTH,HEIGHT);
  g_HTML->Fl_Window::begin();

  g_History.setAutoDelete(1);

  LoadPage("file:///usr/nanox/viewml/index.html");
  
  while(Fl::wait()) {}

  return 0;
}

Fl_Button *Forward=(Fl_Button *)0;


/* XPM */
static char * vml_xpm[] = {
"20 20 199 2",
"  	c None",
". 	c #C12E52",
"+ 	c #7D183B",
"@ 	c #D5116E",
"# 	c #DC0070",
"$ 	c #DA0070",
"% 	c #1F1D1E",
"& 	c #D70070",
"* 	c #D40071",
"= 	c #CF006F",
"- 	c #C9006C",
"; 	c #460025",
"> 	c #CF0072",
", 	c #CD006F",
"' 	c #CD0071",
") 	c #B70063",
"! 	c #5E183F",
"~ 	c #4C1935",
"{ 	c #3F1A2E",
"] 	c #85004B",
"^ 	c #0C0007",
"/ 	c #C30270",
"( 	c #281C23",
"_ 	c #311B28",
": 	c #9E2380",
"< 	c #843788",
"[ 	c #7C3B8A",
"} 	c #68347A",
"| 	c #2C709B",
"1 	c #000203",
"2 	c #0C2D3D",
"3 	c #000304",
"4 	c #0081A4",
"5 	c #004558",
"6 	c #000B0E",
"7 	c #02303E",
"8 	c #008EB3",
"9 	c #008EB1",
"0 	c #00576C",
"a 	c #003947",
"b 	c #001014",
"c 	c #000C0F",
"d 	c #00080A",
"e 	c #008FB0",
"f 	c #0091AE",
"g 	c #0089A5",
"h 	c #00697E",
"i 	c #003E4B",
"j 	c #003844",
"k 	c #003742",
"l 	c #00181D",
"m 	c #001216",
"n 	c #000E11",
"o 	c #0091AD",
"p 	c #0092AD",
"q 	c #0092AC",
"r 	c #0090AA",
"s 	c #008FA9",
"t 	c #008EA7",
"u 	c #008BA6",
"v 	c #008CA5",
"w 	c #0088A1",
"x 	c #0089A1",
"y 	c #00849C",
"z 	c #00859C",
"A 	c #00849B",
"B 	c #00839A",
"C 	c #007F96",
"D 	c #00798F",
"E 	c #00788E",
"F 	c #00778C",
"G 	c #007488",
"H 	c #006A7D",
"I 	c #006476",
"J 	c #006273",
"K 	c #005F70",
"L 	c #00596A",
"M 	c #004A58",
"N 	c #004854",
"O 	c #004552",
"P 	c #00414D",
"Q 	c #003A45",
"R 	c #003A44",
"S 	c #00343D",
"T 	c #002E36",
"U 	c #002D35",
"V 	c #002A31",
"W 	c #002930",
"X 	c #00262D",
"Y 	c #002026",
"Z 	c #001215",
"` 	c #0098AF",
" .	c #0096AE",
"..	c #0098AE",
"+.	c #001316",
"@.	c #000E10",
"#.	c #000708",
"$.	c #0A8191",
"%.	c #009DB0",
"&.	c #0B7A88",
"*.	c #10828E",
"=.	c #084045",
"-.	c #16909B",
";.	c #041111",
">.	c #2D7B70",
",.	c #1F4F48",
"'.	c #060F0D",
").	c #19352C",
"!.	c #3C7D67",
"~.	c #010201",
"{.	c #050705",
"].	c #040502",
"^.	c #5E642F",
"/.	c #6C6F32",
"(.	c #5B5C26",
"_.	c #020200",
":.	c #010100",
"<.	c #030301",
"[.	c #0D0D05",
"}.	c #2C280B",
"|.	c #433D15",
"1.	c #706624",
"2.	c #83792C",
"3.	c #27261F",
"4.	c #282720",
"5.	c #847623",
"6.	c #463F15",
"7.	c #4D461A",
"8.	c #5A532C",
"9.	c #39320E",
"0.	c #453C12",
"a.	c #2A250B",
"b.	c #8A7A25",
"c.	c #A18C2C",
"d.	c #CBB138",
"e.	c #948129",
"f.	c #8E7C27",
"g.	c #827324",
"h.	c #776821",
"i.	c #65581C",
"j.	c #504616",
"k.	c #3E3611",
"l.	c #28230B",
"m.	c #97842A",
"n.	c #524817",
"o.	c #4B4215",
"p.	c #322C0E",
"q.	c #C1A937",
"r.	c #BAA335",
"s.	c #473E14",
"t.	c #E5C842",
"u.	c #BDA536",
"v.	c #B8A135",
"w.	c #A89330",
"x.	c #544A18",
"y.	c #3F3712",
"z.	c #110F05",
"A.	c #675E2F",
"B.	c #4F4825",
"C.	c #312A0C",
"D.	c #574B16",
"E.	c #6A5B1B",
"F.	c #827022",
"G.	c #605219",
"H.	c #AE962E",
"I.	c #534716",
"J.	c #9C872A",
"K.	c #877324",
"L.	c #554917",
"M.	c #161306",
"N.	c #C7AD37",
"O.	c #B8A033",
"P.	c #413812",
"Q.	c #1D1908",
"R.	c #AC9530",
"S.	c #5A4E19",
"T.	c #2B250C",
"U.	c #584C19",
"V.	c #070602",
"W.	c #26210B",
"X.	c #2C260D",
"Y.	c #1E1A09",
"Z.	c #665B2D",
"`.	c #6C6130",
" +	c #554D2B",
".+	c #3A3624",
"++	c #131005",
"@+	c #6B5B1D",
"#+	c #4A4328",
"$+	c #28261F",
"%+	c #050401",
"&+	c #060502",
"*+	c #0A0803",
"=+	c #834830",
"-+	c #230F0E",
";+	c #383838",
">+	c #1D1D1D",
",+	c #161616",
"'+	c #000000",
";+,+>+>+>+>+>+>+>+>+>+>+>+>+>+% _ ( ,+;+",
">+'+'+'+'+'+'+'+'+'+'+z.a.}.-+] , ) ; >+",
">+'+'+'+'+'+'+'+<.T.s.E.(.B.. # & > - { ",
">+'+'++.Q R a ).I.S.++'+;.7 } [ : * = ! ",
">+'+'+X q q e >.[.'+'+'+0 p ` %.< $ ' ~ ",
">+'+'+b u q q h '+'+'+@.A q ..| / @ + >+",
">+'+'+{.*.o q F 3 '+'+N q q  .2 ^ =+5.>+",
">+'+'+9.!.9 q B #.'+1 G q q E d '+K.b.>+",
">+'+l.7.=.f q v Z '+U q q q j '+'+c.h.>+",
">+&+G.].W o q q V '+I q q D 6 '+V.q.P.>+",
">+k.D.'+l t q q P #.x q q k '+'+L.r.<.>+",
">+i.M.'+d y q q M T q q C c '+_.v.x.'+>+",
".+e.:.'+'+J q q K H q q i '+'+@+w.'+'+>+",
"8.f.'+'+'+O q q w s q z m '+C.u.W.'+'+>+",
"A.m.'+'+'+S q q q q q 5 '+Y.R.y.'+'+'+>+",
" +O.%+'+'+Y r q q q 4 '.p.J.X.'+'+'+'+>+",
"$+d.U.'+'+n g f f 8 ,.|.F.Q.'+'+'+'+'+>+",
">+j.t.o.'+~.L &.$.-./.0.*+'+'+'+'+'+'+>+",
">+'+n.N.H.g.^.2.1.6.++'+'+'+'+'+'+'+'+>+",
";+,+>+4. +`.Z.#+3.>+>+>+>+>+>+>+>+>+,+;+"};

static Fl_Pixmap pixmap_vml(vml_xpm);

static unsigned char *image_foo[] = {
(unsigned char*)
"21 22 10 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #838183",
(unsigned char*)
"@\tc #00FF00",
(unsigned char*)
"#\tc #008183",
(unsigned char*)
"$\tc #FFFFFF",
(unsigned char*)
"%\tc #008200",
(unsigned char*)
"&\tc #003439",
(unsigned char*)
"*\tc #202020",
(unsigned char*)
"=\tc #4199A4",
(unsigned char*)
".....................",
(unsigned char*)
"........+@...........",
(unsigned char*)
"........#$@..........",
(unsigned char*)
".......+%.$@.........",
(unsigned char*)
".......&%@.$@........",
(unsigned char*)
"......+&%.@.$@.......",
(unsigned char*)
"......#*%@.@.$@......",
(unsigned char*)
".....+**%.@.@.$@.....",
(unsigned char*)
".....&.@.@.@.@.$@....",
(unsigned char*)
"....&#@.@.@.@.@.$@...",
(unsigned char*)
"....**.@.@.@.@.@.$=..",
(unsigned char*)
"....**@.@.@.@.@.@$=..",
(unsigned char*)
"....&*###########%...",
(unsigned char*)
"....&*%%%%%#%#%=%....",
(unsigned char*)
"....#&%%%%%%=%=%.....",
(unsigned char*)
"....+&%%%%%=%=%......",
(unsigned char*)
".....****%%%=%.......",
(unsigned char*)
".++++#&&&%%=%+++.....",
(unsigned char*)
"+++++++&&%=&+++......",
(unsigned char*)
".++++++&&=&++........",
(unsigned char*)
"....++++&&++.........",
(unsigned char*)
"....................."
};
static Fl_Pixmap pixmap_foo(image_foo);

Fl_Button *Backward=(Fl_Button *)0;

static unsigned char *image_back[] = {
(unsigned char*)
"21 22 10 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #00FF00",
(unsigned char*)
"@\tc #838183",
(unsigned char*)
"#\tc #FFFFFF",
(unsigned char*)
"$\tc #008183",
(unsigned char*)
"%\tc #008200",
(unsigned char*)
"&\tc #003439",
(unsigned char*)
"*\tc #202020",
(unsigned char*)
"=\tc #4199A4",
(unsigned char*)
".....................",
(unsigned char*)
"...........+@........",
(unsigned char*)
"..........+#$........",
(unsigned char*)
".........+#.%@.......",
(unsigned char*)
"........+#.+%&.......",
(unsigned char*)
".......+#.+.%&@......",
(unsigned char*)
"......+#.+.+%*$......",
(unsigned char*)
".....+#.+.+.%**@.....",
(unsigned char*)
"....+#.+.+.+.+.&.....",
(unsigned char*)
"...+#.+.+.+.+.+$&....",
(unsigned char*)
"..=#.+.+.+.+.+.**....",
(unsigned char*)
"..=#+.+.+.+.+.+**....",
(unsigned char*)
"...%$$$$$$$$$$$*&....",
(unsigned char*)
"....%=%$%$%%%%%*&....",
(unsigned char*)
".....%=%=%%%%%%&$....",
(unsigned char*)
"......%=%=%%%%%&@....",
(unsigned char*)
".......%=%%%****.....",
(unsigned char*)
".....@@@%=%%&&&$@@@@.",
(unsigned char*)
"......@@@&=%&&@@@@@@@",
(unsigned char*)
"........@@&=&&@@@@@@.",
(unsigned char*)
".........@@&&@@@@....",
(unsigned char*)
"....................."
};
static Fl_Pixmap pixmap_back(image_back);

Fl_Button *Stop=(Fl_Button *)0;

static unsigned char *image_stop[] = {
(unsigned char*)
"21 21 10 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #6265CD",
(unsigned char*)
"@\tc #838183",
(unsigned char*)
"#\tc #9C99FF",
(unsigned char*)
"$\tc #313062",
(unsigned char*)
"%\tc #000000",
(unsigned char*)
"&\tc #830000",
(unsigned char*)
"*\tc #FF0000",
(unsigned char*)
"=\tc #202020",
(unsigned char*)
".....................",
(unsigned char*)
"....+++++++++++@.....",
(unsigned char*)
"...#@@#@#@#@#@+$.....",
(unsigned char*)
".$#+##@$%$@.@.@%%%$..",
(unsigned char*)
".$.+.@&&&&%@#.$@+%$..",
(unsigned char*)
"..$+.&&*&&$%@.@+%&%..",
(unsigned char*)
"..$+..***&$%#.+.&&...",
(unsigned char*)
"...+...**&$%@.@$&&...",
(unsigned char*)
"...+.#.*&=%=#.+$.&...",
(unsigned char*)
".$@+.##====+@.$$$$$..",
(unsigned char*)
".$#+.#%%%%+$#.+@+%$..",
(unsigned char*)
"..$+.@$$++$$@.@+%$$..",
(unsigned char*)
"...+.%@@$$$$#.+.$$...",
(unsigned char*)
"...+.%@@$$=@@.@$$$...",
(unsigned char*)
".$#+.#@$$$$#@.@$$$%..",
(unsigned char*)
".$#+.@%%%%=@#.$@+%$..",
(unsigned char*)
".@$+.%====%$@.++%$$..",
(unsigned char*)
"..$+.%$$$=$%#.+@$$...",
(unsigned char*)
"...+..$$=$=%@.+$$$@@.",
(unsigned char*)
"...+###===$@#@+=@@@@.",
(unsigned char*)
".....$$$+++++$+%@@@.."
};
static Fl_Pixmap pixmap_stop(image_stop);

Fl_Button *Reload=(Fl_Button *)0;

static unsigned char *image_reload[] = {
(unsigned char*)
"21 21 9 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #6265CD",
(unsigned char*)
"@\tc #313062",
(unsigned char*)
"#\tc #FFFFFF",
(unsigned char*)
"$\tc #9C99FF",
(unsigned char*)
"%\tc #202020",
(unsigned char*)
"&\tc #000083",
(unsigned char*)
"*\tc #838183",
(unsigned char*)
".....................",
(unsigned char*)
".....++++++++@.......",
(unsigned char*)
"....+#+$+$++%&@@.....",
(unsigned char*)
"...+#$$$$$+%%%&@@....",
(unsigned char*)
"#....$.$.$.$$$+%%@...",
(unsigned char*)
"+#..$.$.$.$.$%%%@@...",
(unsigned char*)
".+#..$.$.$.$%@&&@@...",
(unsigned char*)
"..+#......+%@&&&@@...",
(unsigned char*)
"...+#....+%@&&&&@@...",
(unsigned char*)
"....+#..+%@+&&&&@@...",
(unsigned char*)
".....+#+%@.+&&&&@@...",
(unsigned char*)
"......@@@..+&&&%@@...",
(unsigned char*)
"...........+&&&%@@.*.",
(unsigned char*)
"...........$$&%%@@.**",
(unsigned char*)
"...$$$$$$$$$+%&%@@***",
(unsigned char*)
"...$$$$$$$$$+%%%@@++.",
(unsigned char*)
"...$+$+$+$+$++%@+@+*.",
(unsigned char*)
"...$++++++++++++@+**.",
(unsigned char*)
"....$++@@@@@&@@@+**..",
(unsigned char*)
".......***********...",
(unsigned char*)
"....................."
};

static Fl_Pixmap pixmap_reload(image_reload);

//davet
#define MAX_PROTO_STRLEN 7 // must be set to length of longest protocol
static char *protocols[] = 
{
  "http://",
  "file://",
#ifdef OPTIONAL_URI_PROTOCOLS
  "ftp://",
  "afs://",
  "news:",
  "nntp:",
  "mid:",
  "cid:",
  "mailto:",
  "wais://",
  "prospero://",
  "telnet://"
  "gopher://",
#endif
  NULL
};

void _LoadPage(const char * URL)
{
  char buf[1024];
  int pindex;

  ResizeScrollbars(0,0);

  // get URL protocol
  strncpy(buf,URL,MAX_PROTO_STRLEN);
  buf[MAX_PROTO_STRLEN] = 0x00;

  // check for protocol in list
  pindex = 0;
  while(protocols[pindex] != NULL)
  {
    if(strncmp(buf,protocols[pindex],strlen(protocols[pindex])) == 0)
      break;
    ++pindex;
  }

  // check for missing protocol
  if(protocols[pindex] == NULL)
    sprintf(buf,"http://%s",URL);
  else
    strcpy(buf,URL);

  g_Input->value(buf);
  g_Input->set_changed();
  g_Input->mark(256);
  
  g_Conn->loadPage(buf);
}

void LoadPage(const char * URL)
{
  if(g_HistoryIndex != -1 && g_HistoryIndex != g_History.count()-1) {
    while(g_HistoryIndex != g_History.count()-1)
      g_History.removeLast();
  }

  g_History.append(new QString(URL));
  g_HistoryIndex = g_History.count() - 1;
  _LoadPage(URL);

}

void LoadDoc(const char * URL, int w, int h) 
{
  if(URL) {
    LoadPage(URL);
  }

  ResizeScrollbars(w,h);
}

void ResizeScrollbars(int w, int h)
{
  g_VertScroll->value(0.0,HEIGHT,0.0,(double)(h - HEIGHT) + HEIGHT);
  g_HorzScroll->value(0.0,WIDTH,0.0,(double)(w - WIDTH ) + WIDTH);
}

void Reload_Callback(Fl_Widget * w, void * data)
{
  _LoadPage(g_Input->value());
}

void Forward_Callback(Fl_Widget * w, void * data)
{
  if(g_HistoryIndex == -1 || g_HistoryIndex+1 > g_History.count()-1)
    return;

  _LoadPage(*g_History.at(++g_HistoryIndex));
}

void Backward_Callback(Fl_Widget * w, void * data)
{
  if(g_HistoryIndex == -1 || g_HistoryIndex == 0)
    return;

  _LoadPage(*g_History.at(--g_HistoryIndex));
}

static void VScroll_Callback(Fl_Widget* o, void*) {
  //  printf("%g\n", ((Fl_Valuator*)o)->value());
  g_HTML->slotScrollVert(((Fl_Valuator*)o)->value());
}

static void HScroll_Callback(Fl_Widget* o, void*) {
  //  printf("%g\n", ((Fl_Valuator*)o)->value());
  g_HTML->slotScrollHorz(((Fl_Valuator*)o)->value());
}

void Input_Callback(Fl_Widget * w, void * data)
{
  Fl_Input * i = (Fl_Input*)w;
  QString str  =  i->value();
  History hobj;

  // if old and new strings match, must have been a CR
  if(strcmp((char *)str,oldstr) == 0)
  {
    oldstr[0] = 0x00;
    hobj.AddToHistoryList(str);
    hindex = 1;
    LoadPage(str);
  }

  // if there's a space in str, set a history entry
  else if(strstr((char *)str," "))
  {
    i->value(hobj.GetHistoryEntry(hindex));
    i->set_changed();
    i->mark(256);
    strcpy(oldstr,hobj.GetHistoryEntry(hindex));
    if(hobj.GetHistoryEntry(++hindex) == NULL)
      hindex = 0;
  }

  // must be an updated string - save it
  else
  {
    strcpy(oldstr,(char *)str);
  }
}

void ResizeWindow(int w, int h)
{
  g_AppWidth = w;
  g_AppHeight = h;

  g_AppWindow->resize(g_AppWindow->x(), g_AppWindow->y(), w,h);
  g_VertScroll->resize(WIDTH + SCROLLPAD + SCROLLPAD, TITLEHEIGHT,SCROLLWIDTH,HEIGHT);
  g_HorzScroll->resize(SCROLLPAD, HEIGHT + TITLEHEIGHT + SCROLLPAD, WIDTH,SCROLLHEIGHT);
  g_HTML->setGeometry(SCROLLPAD,TITLEHEIGHT,WIDTH,HEIGHT);

  g_HTML->redraw();
}

Fl_Window* make_window() 
{
  char buf[256];
  Fl_Window* w;
  { Fl_Window* o = new Fl_Window(APPWIDTH, APPHEIGHT);
  g_AppWindow = o;
    w = o;
    { 
      Fl_Button* o = Forward = new Fl_Button(30, 3, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      o->callback(Forward_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_foo.label(o);
    }
    { 
      Fl_Button* o = Backward = new Fl_Button(5, 3, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      o->callback(Backward_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_back.label(o);
    }
/*
    { 
      Fl_Button* o = Stop = new Fl_Button(55, 3, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      pixmap_stop.label(o);
    }
*/
    { 
      Fl_Button* o = Reload = new Fl_Button(54, 3, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      o->callback(Reload_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_reload.label(o);
    }
    { 
      Fl_Input * o = new Fl_Input(81, 5, APPWIDTH - 110 , 20, "");
      o->callback(Input_Callback,0);
      o->when(FL_WHEN_ENTER_KEY_CHANGED);
      g_Input = o;
    }
    
    {
      Fl_Button* o = new Fl_Button(APPWIDTH -24 ,5,20,20);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_OFLAT_BOX);
      pixmap_vml.label(o);
    }

    { 
      g_VertScroll = new Fl_Scrollbar(WIDTH + SCROLLPAD + SCROLLPAD, TITLEHEIGHT,
				      SCROLLWIDTH,HEIGHT);
      g_VertScroll->callback((Fl_Callback*)VScroll_Callback);

      g_HorzScroll = new Fl_Scrollbar(SCROLLPAD, HEIGHT + TITLEHEIGHT + SCROLLPAD, 
				      WIDTH,
				      SCROLLHEIGHT);
      g_HorzScroll->callback((Fl_Callback*)HScroll_Callback);
      g_HorzScroll->type(FL_HORIZONTAL);

      HTML_WIDGET * v;
      v = new HTML_WIDGET();
      v->setGeometry(SCROLLPAD,TITLEHEIGHT,WIDTH,HEIGHT);
      
      g_Conn->Connect(v);
      g_HTML = v;
    }

    o->end();
  }

  // recover last entry from history and display the page
  History hobj;

  return w;
}





